import globalPluginHandler
from scriptHandler import script
from logHandler import log
import ui
import speech
import os
import urllib
import urllib.request
import urllib.parse
from urllib.request import urlopen
import locale
import wx
from .xml.etree import ElementTree as ET
import http.cookiejar
import re
import threading
import time
import gui
from gui import guiHelper
from os.path import expanduser
from . import csv
import datetime
import winsound
import tones
from NVDAObjects.IAccessible import IAccessible, getNVDAObjectFromEvent
import api
import controlTypes
import webbrowser
import json
import addonHandler
from .bs4 import BeautifulSoup
import requests
from speech import sayAll
from collections import defaultdict

locale.setlocale(locale.LC_ALL, "czech")

RssFolder = os.path.join(os.getenv('APPDATA'), "Marsus", "NVDA", "Rss")
articlesFolder = os.path.join(RssFolder, "Articles")
archiveFolder = os.path.join(RssFolder, "Archive")
opmlFilePath = os.path.join(RssFolder, "channels.opml")
settingsFilePath = os.path.join(RssFolder, "settings.json")
articlesHeader = ['title', 'description', 'date', 'link', 'deleted']
mainTitle = "RSS čtečka"
soundPath = os.path.join(os.path.dirname(__file__), "sound")
currentArticleDescription = ""
currentChannelNewMessages = ""
currentChannelResults = ""
openArticlesAsText = False
askArticleDeletion = False
askArticleArchivation = True
automaticallyAnnounceNumberofNews = True
automaticallyAnnounceDescription = True
newArticlesAfterOpening = True
downloadArticlesAfterStartup = False
downloadArticlesInterval = 0
searchFilter = False
inAllArticles = False
inProcessOfReadingNewArticles = False

_addonDir = os.path.join(os.path.dirname(__file__), "..", "..")
_curAddon = addonHandler.Addon(_addonDir)
_addonSummary = _curAddon.manifest['summary']
_addonVersion = _curAddon.manifest['version']
_addonPage = _curAddon.manifest['url']

def validFileName(s):
  validName = ""
  for i in s:
    if i in "\/:*?<>|":
      i = "-"
    validName = validName + i
  return validName

def playSound(fSoundName, looped=False):
  fSound = os.path.join(soundPath, fSoundName)
  if os.path.isfile(fSound):
    if looped:
      winsound.PlaySound(fSound, winsound.SND_FILENAME + winsound.SND_LOOP + winsound.SND_ASYNC)
    else:
      winsound.PlaySound(fSound, winsound.SND_FILENAME)

def stopsound():
      winsound.PlaySound(None, winsound.SND_PURGE)

def convertDownloadArticlesInterval(i):
  if i == 1: # 5 minut
    return 300000
  elif i == 2: # 10 minut
    return 600000
  elif i == 3: # 15 minut
    return 900000
  elif i == 4: # 30 minut
    return 1800000
  elif i == 5: # 1 hodina
    return 3600000
  elif i == 6: # 2 hodiny
    return 7200000
  elif i == 7: # 3 hodiny
    return 10800000
  else:
    return 0


class TextWindow(wx.Frame):

  dispText = ""
  dlgTitle = ""
  sayAllRunning = False

  def __init__(self, parent):
    # the title of text window
    super(TextWindow, self).__init__(parent, size=(1000, 800), title=self.dlgTitle)
    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
      
    # menu items
    item = menu.Append(wx.ID_ANY, "Spustit nebo zastavit plynulé čtení\tMezerník")
    self.Bind(wx.EVT_MENU, self.startstopReading, item)
    item = menu.Append(wx.ID_ANY, "Zobrazit zprávu ve webovém prohlížeči\tF5")
    self.Bind(wx.EVT_MENU, self.displayinBrowserItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "&Zavřít")
    self.Bind(wx.EVT_MENU, self.onClose, item)

    # the menu title
    menuBar.Append(menu, "Čtečka")
    self.SetMenuBar(menuBar)

    # the edit field
    self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, style=wx.TE_MULTILINE | wx.TE_READONLY)
    self.outputCtrl.Bind(wx.EVT_KEY_UP, self.onOutputKeyUp)
    sHelper.addItem(self.outputCtrl, proportion=1, flag=wx.EXPAND)

    self.outputCtrl.SetValue(self.dispText)
    self.outputCtrl.SetFocus()

  def startReading(self):
    speech.cancelSpeech()
    if not self.sayAllRunning:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def startstopReading(self, evt):
    if self.sayAllRunning:
      self.sayAllRunning = False
      sayAll.SayAllHandler.stop()
    else:
      sayAll.SayAllHandler.readText(sayAll.CURSOR.CARET)
      self.sayAllRunning = True

  def displayinBrowserItem(self, evt):
    index = self.Parent.getSelectedArticleIndex()
    articleAddress = self.Parent.Parent.channel.getArticleLink(index)
    os.startfile(articleAddress)
    self.Close()

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()

  def onOutputKeyUp(self, evt):
    key = evt.GetKeyCode()
    if (key == wx.WXK_ESCAPE) or (key == wx.WXK_BACK):
      self.Close()
    elif key != wx.WXK_SPACE:
      self.sayAllRunning = False
    evt.Skip()


class RssReader(wx.Frame):

  appdataFolder = os.getenv('APPDATA')
  opmlTree = None
  channel = None
  newChannelName = None
  dbSelectedUrl = ""
  inArchive = False
  archiveTitle = "Archiv"
  newsChannels = dict()
  filteredNews = False
  searchResults = dict()
  displayAllChannelsTitle = "Zobrazit všechny kanály"
  displayAllMessagesTitle = "Vše"
  if not os.path.exists(os.path.join(appdataFolder, "Marsus")):
    os.mkdir(os.path.join(appdataFolder, "Marsus"))
  if not os.path.exists(os.path.join(appdataFolder, "Marsus", "NVDA")):
    os.mkdir(os.path.join(appdataFolder, "Marsus", "NVDA"))
  if not os.path.exists(RssFolder):
    os.mkdir(RssFolder)
  if not os.path.exists(articlesFolder):
    os.mkdir(articlesFolder)
  # create OPML file
  if not os.path.isfile(opmlFilePath):
    try:
      opml = ET.Element("opml")
      opml.set("version", "2.0")
      head = ET.Element("head")
      title = ET.Element("title")
      title.text = mainTitle
      head.append(title)
      opml.append(head)
      body = ET.Element("body")
      opml.append(body)
      with open(opmlFilePath, 'wb') as f:
        f.write(ET.tostring(opml, encoding='utf-8', method='xml'))
    except Exception as e:
      raise e

  _instance = None

  def __new__(cls, *args, **kwargs):
    # Make this a singleton.
    if RssReader._instance is None:
      return super(RssReader, cls).__new__(cls, *args, **kwargs)
    return RssReader._instance

  def __init__(self, parent):
    if RssReader._instance is not None:
      return
    RssReader._instance = self

    # The window title
    super(RssReader, self).__init__(parent, size=(1000, 800), title=mainTitle)

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.choices = []

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "&Nový kanál\tCtrl+N")
    self.Bind(wx.EVT_MENU, self.newChannelItem, item)
    item = menu.Append(wx.ID_ANY, "Přidat kanál z &databáze\tCtrl+D")
    self.Bind(wx.EVT_MENU, self.channelFromDatabaseItem, item)
    item = menu.Append(wx.ID_ANY, "&Vlastnosti kanálu\tAlt+Enter")
    self.Bind(wx.EVT_MENU, self.channelPropertiesItem, item)
    item = menu.Append(wx.ID_ANY, "&Smazat kanál\tDelete")
    self.Bind(wx.EVT_MENU, self.deleteChannelItem, item)
    item = menu.Append(wx.ID_ANY, "&Importovat kanály ze souboru OPML")
    self.Bind(wx.EVT_MENU, self.importChannelsFromOpmlItem, item)
    item = menu.Append(wx.ID_ANY, "&Exportovat kanály do souboru OPML")
    self.Bind(wx.EVT_MENU, self.exportChannelsToOpmlItem, item)
    item = menu.Append(wx.ID_ANY, "N&ačíst nové zprávy ze všech kanálů\tF5")
    self.Bind(wx.EVT_MENU, self.readAllNewArticlesItem, item)
    item = menu.Append(wx.ID_ANY, "N&ačíst nové zprávy z vybraného kanálu\tCtrl+F5")
    self.Bind(wx.EVT_MENU, self.readSelectedNewArticlesItem, item)
    self.newsMenuItem = menu.AppendCheckItem(wx.ID_ANY, "Zobrazit &pouze kanály s novými zprávami\tF4")
    self.Bind(wx.EVT_MENU, self.displayChannelsFilterItem, self.newsMenuItem)
    item = menu.Append(wx.ID_ANY, "&Otevřít seznam zpráv\tEnter")
    self.Bind(wx.EVT_MENU, self.openArticlesListItem, item)
    item = menu.Append(wx.ID_ANY, "Ohlásit &počet zpráv ve vybraném kanálu\tF2")
    self.Bind(wx.EVT_MENU, self.sayArticlesNumberItem, item)
    item = menu.Append(wx.ID_ANY, "&Hledat text ve zprávách všech kanálů\tCtrl+F")
    self.Bind(wx.EVT_MENU, self.searchItem, item)
    item = menu.Append(wx.ID_ANY, "Otevřít &archiv\tCtrl+F6")
    self.Bind(wx.EVT_MENU, self.openArchiveItem, item)
    item = menu.Append(wx.ID_ANY, "&Zpět z archivu\tBackspace")
    self.Bind(wx.EVT_MENU, self.backFromArchiveItem, item)
    item = menu.Append(wx.ID_ANY, "N&astavení\tF12")
    self.Bind(wx.EVT_MENU, self.settingsItem, item)
    item = menu.Append(wx.ID_ANY, "Nápo&věda")
    self.Bind(wx.EVT_MENU, self.helpItem, item)
    item = menu.Append(wx.ID_ANY, "&O programu")
    self.Bind(wx.EVT_MENU, self.aboutItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, mainTitle)
    self.SetMenuBar(menuBar)

    # load config
    self.readcfg()

    # load OPML file
    self.loadOpmlChannels()

    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList = wx.ListBox(self, choices=self.choices)
    self.channelsList.Bind(wx.EVT_LISTBOX, self.onchannelsListChange)
    self.channelsList.Bind(wx.EVT_LISTBOX_DCLICK, self.onchannelsListChoice)
    self.channelsList.Bind(wx.EVT_KEY_UP, self.onchannelsListKeyUp)
    self.channelsList.SetMinSize((1000,800));
    sHelper.addItem(self.channelsList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.channelsList.SetFocus()
    self.CentreOnScreen()
    if self.channelsList.GetCount() > 0:
      self.channelsList.Selection = 0
      if downloadArticlesAfterStartup:
        # the timer which will start downloading new messages shortly after startup
        self.timer = wx.Timer(self)
        self.timer.StartOnce(2000)
        self.Bind(wx.EVT_TIMER, self.readAllNewArticlesItem)
    else:
      wx.CallAfter(self.noChannels)
    self.addArchiveToList()
    # the timer which will download new messages in regular interval
    self.regularTimer = wx.Timer(self)
    self.Bind(wx.EVT_TIMER, self.readAllNewArticlesItem)
    if downloadArticlesInterval > 0:
      self.regularTimer.Start(convertDownloadArticlesInterval(downloadArticlesInterval))

  def searchItem(self, evt):
    if self.channelsList.GetCount() == 0:
      return
    global searchFilter
    with wx.TextEntryDialog(
      self, "Zadejte hledaný text:",
      "Hledání textu ve zprávách všech kanálů", value=""
    ) as d:
      if d.ShowModal() == wx.ID_CANCEL or not d.Value:
        return
      # searching in all channels and articles
      self.searchResults.clear()
      for file in os.listdir(articlesFolder):
        curFile = os.path.join(articlesFolder, file)  
        if os.path.isfile(curFile):
          artResultList = []
          with open(curFile, 'r', encoding="utf8", newline='') as f:
            reader = csv.DictReader(f)
            for row in reader:
              # when searched string will be in title or in description
              if (d.Value.upper() in row["title"].upper()) or (d.Value.upper() in row["description"].upper()):
                # and when the message is not deleted
                if row["deleted"] == "0":
                  artResultList.append(row["title"])
          if len(artResultList) > 0:
            chfile = os.path.splitext(file)[0]
            self.searchResults[chfile] = artResultList
      if len(self.searchResults) == 0:
        dlgMessage = "%s se v žádné zprávě nevyskytuje." % d.Value
        dlgTitle = "Upozornění"
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        return
    searchFilter = True
    self.channelsList.Clear()
    searchChannelsList = []
    for key, value in self.searchResults.items():
      searchChannelsList.append(key)  
    searchChannelsList.sort(key=locale.strxfrm)
    searchChannelsList.insert(0, self.displayAllChannelsTitle)
    self.channelsList.InsertItems(searchChannelsList, 0)
    self.channelsList.Selection = 0
    self.onchannelsListChange(None)
    self.channelsList.SetFocus()

  def exportChannelsToOpmlItem(self, evt):
    filename = wx.FileSelector(
      "Uložit jako", default_filename="rss.opml", flags=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT, parent=self
    )
    if filename:
      with open(filename, 'wb') as f:
        f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
      self.channelsList.SetFocus()

  def importChannelsFromOpmlItem(self, evt):
    with wx.FileDialog(
      self, "Import ze souboru OPML",
      wildcard=_("Soubory OPML (*.opml)|*.opml"),
      style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
    ) as importDialog:
      if importDialog.ShowModal() == wx.ID_CANCEL:
        return
      pathName = importDialog.GetPath()
    try:
      importOpmlTree = ET.parse(pathName)
    except Exception as e:
      raise e
    feeds = importOpmlTree.getroot().findall("./body/outline")
    body = self.opmlTree.getroot().find("body")
    chList = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    for feed in feeds:
      if not feed.get("xmlUrl"):
        # probably older format OPML than 2.0
        for subfeed in feed.iter("outline"):
          if subfeed.get("xmlUrl"):
            # when channel name is specified
            if subfeed.get("text"):
              # and is not stored in title attribute
              if not subfeed.get("title"):
                # we set the same name for title as it is in the text attribute
                subfeed.set("title", subfeed.get("text"))
              chname = subfeed.get("title")
              # check for channel name duplicity
              if chname in chList:
                dlgMessage = "Kanál %s se již v seznamu kanálů vyskytuje." % chname
                dlgTitle = "Upozornění"
                wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
              else:
                body.append(subfeed)
            else:
              # when the name is not present, we create channel as it is manually inserted (with existence check)
              self.createChannel(subfeed.get("xmlUrl"))
      else:
        # OPML version 2.0
        chname = feed.get("title")
        # check for channel name duplicity
        if chname in chList:
          dlgMessage = "Kanál %s se již v seznamu kanálů vyskytuje." % chname
          dlgTitle = "Upozornění"
          wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        else:
          body.append(feed)
    with open(opmlFilePath, 'wb') as f:
      f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.Clear()
    for choice in self.choices:
      self.channelsList.Append(choice)
    self.addArchiveToList()
    self.channelsList.Selection = self.channelsList.GetCount() - 1
    self.onchannelsListChange(None)
    self.channelsList.SetFocus()
    if self.channelsList.GetCount() > 0:
      self.channelsList.Selection = 0
    # switch off filter of channels with news
    self.filteredNews = False
    self.newsMenuItem.Check(False)

  def displayChannelsFilterItem(self, evt):
    if self.inArchive:
      return
    if self.filteredNews:
      # switch filter off, display all channels
      self.filteredNews = False
      # obtain channels from OPML
      self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
      self.choices.sort(key=locale.strxfrm)
      # assign channels to list items
      self.channelsList.Clear()
      for choice in self.choices:
        self.channelsList.Append(choice)
      self.addArchiveToList()
      self.channelsList.Selection = 0
      self.onchannelsListChange(None)
      self.channelsList.SetFocus()
    else:
      # display only channels with new messages
      if len(self.newsChannels) == 0:
        self.newsMenuItem.Check(False)
        ui.message("Žádné kanály s nepřečtenými zprávami.")
      else:
        self.filteredNews = True
        self.channelsList.Clear()
        newChannelsList = []
        for key, value in self.newsChannels.items():
          newChannelsList.append(key)  
        # newChannelsList = list(self.newsChannels.keys()), 0
        newChannelsList.sort(key=locale.strxfrm)
        self.channelsList.InsertItems(newChannelsList, 0)
        self.channelsList.Selection = 0
        self.onchannelsListChange(None)
        self.channelsList.SetFocus()

  def helpItem(self, evt):
    from . import addonDoc
    helpFile = addonDoc.getDocPath("readme.html")
    if helpFile:
      os.startfile(helpFile)
    else:
      os.startfile(os.path.join(_addonDir, "doc", "cs", "readme.html"))

  def readcfg(self):
    global openArticlesAsText
    global askArticleDeletion
    global askArticleArchivation
    global automaticallyAnnounceNumberofNews
    global automaticallyAnnounceDescription
    global newArticlesAfterOpening
    global downloadArticlesAfterStartup
    global downloadArticlesInterval
    if not os.path.isfile(settingsFilePath):
      config = {"OpenArticlesAsText": True, "AskArticleDeletion": True, "AskArticleArchivation": True, "AutomaticallyAnnounceNumberofNews": True, "AutomaticallyAnnounceDescription": True, "NewArticlesAfterOpening": True, "DownloadArticlesAfterStartup": False, "DownloadArticlesInterval": 0}
      SetFile = settingsFilePath
      with open(SetFile, 'w') as f:
        json.dump(config, f)    
    SetFile = settingsFilePath
    with open(SetFile, 'r') as f:
      config = json.load(f)
    try:
      openArticlesAsText = config['OpenArticlesAsText']
    except: pass
    try:
      askArticleDeletion = config['AskArticleDeletion']
    except: pass
    try:
      askArticleArchivation = config['AskArticleArchivation']
    except: pass
    try:
      automaticallyAnnounceNumberofNews = config['AutomaticallyAnnounceNumberofNews']
    except: pass
    try:
      automaticallyAnnounceDescription = config['AutomaticallyAnnounceDescription']
    except: pass
    try:
      newArticlesAfterOpening = config['NewArticlesAfterOpening']
    except: pass
    try:
      downloadArticlesAfterStartup = config['DownloadArticlesAfterStartup']
    except: pass
    try:
      downloadArticlesInterval = config['DownloadArticlesInterval']
    except: pass

  def openArchiveItem(self, evt):
    if self.channelsList.GetString(self.channelsList.GetCount()-1) != self.archiveTitle:
      return
    self.inArchive = True
    self.channelsList.Clear()
    self.choices = []
    for filename in os.listdir(archiveFolder):
      self.choices.append(filename.rsplit(" ", 1)[0])
    self.channelsList.InsertItems(self.choices, 0)
    self.channelsList.Selection = 0
    self.channelsList.SetFocus()

  def aboutItem(self, evt):
    dlgMessage = mainTitle + "\nverze " + _addonVersion + "\nCopyright (c) 2023-2024 Marek Susčík"
    dlgTitle = "Informace o programu"
    wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)

  def settingsItem(self, evt):
    self.Hide()
    self.Disable()
    try:
      RssReaderSettingsDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e

  def backFromArchiveItem(self, evt):
    if not self.inArchive:
      return
    self.inArchive = False
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.Clear()
    for choice in self.choices:
      self.channelsList.Append(choice)
    self.addArchiveToList()
    self.channelsList.Selection = self.channelsList.GetCount() - 1
    self.channelsList.SetFocus()

  def addArchiveToList(self):
    if os.path.exists(archiveFolder):
      self.channelsList.InsertItems([self.archiveTitle], self.channelsList.GetCount())
    if self.channelsList.GetCount() >1:
      self.channelsList.InsertItems([self.displayAllMessagesTitle], 0)
    if self.channelsList.GetCount() >0:
      self.channelsList.Selection = 0

  def noChannels(self):
    if gui.messageBox(
      "V RSS čtečce nemáte žádné kanály. Chcete přidat kanál nyní?",
      "Upozornění",
      wx.YES | wx.NO | wx.ICON_QUESTION, self
    ) == wx.YES:
      if gui.messageBox(
        "Chcete vybrat kanál z databáze připravených kanálů?",
        "Upozornění",
        wx.YES | wx.NO | wx.ICON_QUESTION, self
      ) == wx.YES:
        ChannelDatabaseDialog.passkey = True
        self.channelFromDatabaseItem(None)
      else:
        self.newChannelItem(None)

  def getCurrentChannel(self):
    return self.channel

  def playProgress(self, countProg, allProg):
    curProg = countProg / allProg * 1000
    lastProg = -1
    if lastProg != curProg:
      lastProg = curProg
      tones.beep(curProg, 40)

  def loadOpmlChannels(self):
    try:
      self.opmlTree = ET.parse(opmlFilePath)
    except Exception as e:
      raise e

  def saveOpmlChannel(self, title, url, type):
    element = ET.Element("outline")
    element.set("title", title)
    element.set("text", title)
    element.set("xmlUrl", url)
    element.set("type", type)
    body = self.opmlTree.getroot().find("body")
    body.append(element)
    with open(opmlFilePath, 'wb') as f:
      f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))

  def createChannel(self, address):
    cherror = False
    try:
      channel = RssChannel()
      channel.setUrl(address)
      if channel.readArticles() == -1:
        raise ValueError("Problém při načítání kanálu.")
    except Exception as e:
      wx.CallAfter(
        gui.messageBox,
        'Nemohu přidat kanál z adresy %s: %s' % (address, e),
        ("Chyba"),
        wx.OK | wx.ICON_ERROR
      )
      cherror = True
    if cherror:
      return None
    chName = channel.getChannelName()
    chType = channel.getFeedType()
    chList = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    if chName in chList:
      wx.CallAfter(
        gui.messageBox,
        'Kanál %s se již v seznamu kanálů nachází.' % chName,
        ("Upozornění"),
        wx.OK | wx.ICON_ERROR
      )
      return ""
    self.saveOpmlChannel(chName, address, chType)
    return chName

  def channelFromDatabaseItem(self, evt):
    if self.filteredNews:
      dlgMessage = "Máte zobrazeny pouze kanály s novými zprávami. Pro možnost přidávat kanály, prosím, zobrazte všechny kanály."
      dlgTitle = "Upozornění"
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    self.dbSelectedUrl = ""
    self.Hide()
    self.Disable()
    try:
      ChannelDatabaseDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if not self.dbSelectedUrl:
      return
    name = self.createChannel(self.dbSelectedUrl)
    if name:
      self.choices.append(name)
      self.choices.sort(key=locale.strxfrm)
      self.channelsList.Clear()
      self.channelsList.InsertItems(self.choices, 0)
      self.addArchiveToList()
      chIndex = self.channelsList.FindString(name)
      self.channelsList.Selection = chIndex
      self.channelsList.SetFocus()

  def newChannelItem(self, evt):
    if self.filteredNews:
      dlgMessage = "Máte zobrazeny pouze kanály s novými zprávami. Pro možnost přidávat kanály, prosím, zobrazte všechny kanály."
      dlgTitle = "Upozornění"
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    with wx.TextEntryDialog(
      self, "Zadejte adresu nového kanálu:",
      "Nový kanál"
    ) as d:
      if d.ShowModal() == wx.ID_CANCEL or not d.Value:
        self.channelsList.SetFocus()
        return
    name = self.createChannel(d.Value)
    if name:
      self.choices.append(name)
      self.choices.sort(key=locale.strxfrm)
      self.channelsList.Clear()
      self.channelsList.InsertItems(self.choices, 0)
      self.addArchiveToList()
      chIndex = self.channelsList.FindString(name)
      self.channelsList.Selection = chIndex
      self.channelsList.SetFocus()

  def openArticlesListItem(self, evt):
    self.onchannelsListChoice(self)

  def channelPropertiesItem(self, evt):
    try:
      result = ChannelPropertiesDialog(self).ShowModal()
    except Exception as e:
      self.Enable()
      raise e
    if result:
      # obtain channels from OPML
      self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
      self.choices.sort(key=locale.strxfrm)
      # assign channels to list items
      self.channelsList.Clear()
      for choice in self.choices:
        self.channelsList.Append(choice)
      chIndex = self.channelsList.FindString(self.newChannelName)
      self.channelsList.Selection = chIndex
      self.channelsList.SetFocus()

  def deleteChannelItem(self, evt):
    # some items can not be deleted
    if (self.channelsList.StringSelection == self.archiveTitle) or (self.channelsList.StringSelection == self.displayAllChannelsTitle) or (self.channelsList.StringSelection == self.displayAllMessagesTitle):
      return
    # when only channels with new messages are displayed, delete is not possible
    if self.filteredNews:
      dlgMessage = "Máte zobrazeny pouze kanály s novými zprávami. Pro možnost mazat kanály, prosím, zobrazte všechny kanály."
      dlgTitle = "Upozornění"
      wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
      return
    if gui.messageBox(
      "Chcete smazat kanál " + self.channelsList.StringSelection + "?",
      "Odstranění kanálu",
      wx.YES | wx.NO | wx.ICON_QUESTION, self
    ) == wx.NO:
      self.channelsList.SetFocus()
      return
    self.reallyDeleteChannel()

  def reallyDeleteChannel(self):
    delItem = self.channelsList.StringSelection
    selItemNumber = self.channelsList.GetSelection()
    if not self.inArchive:
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == delItem:
          body = self.opmlTree.getroot().find("body")
          body.remove(outline)
          break
      with open(opmlFilePath, 'wb') as f:
        f.write(ET.tostring(self.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # when Vše is in the list, it is necessary to lower delete index in choices list
    if self.channelsList.Items[0].startswith(self.displayAllMessagesTitle):
      del self.choices[self.channelsList.GetSelection()-1]
    else:
      del self.choices[self.channelsList.GetSelection()]
    # remove file with channel articles
    if self.inArchive:
      # find file to delete
      for filename in os.listdir(archiveFolder):
        if filename.startswith(delItem + " ("):
          break
      articleFile = os.path.join(archiveFolder, filename)
    else:
      articleFile = os.path.join(articlesFolder, delItem + ".txt")
    if os.path.isfile(articleFile):
      os.remove(articleFile)
    # assign channels to list items
    self.channelsList.Clear()
    if len(self.choices) == 0:
      if self.inArchive:
        # delete archive folder
        import shutil
        shutil.rmtree(archiveFolder)
        dlgMessage = "Archiv je prázdný."
        dlgTitle = "Informace"
        wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
        wx.CallAfter(self.backFromArchiveItem, None)
      else:
        self.noChannels()
      return
    for choice in self.choices:
      self.channelsList.Append(choice)
    self.addArchiveToList()
    if selItemNumber == self.channelsList.GetCount():
      self.channelsList.Selection = selItemNumber-1
    else:
      self.channelsList.Selection = selItemNumber
    self.channelsList.SetFocus()

  def readSelectedNewArticlesItem(self, evt):
    if self.inArchive:
      ui.message("Jste v archivu. Pro příjem nových zpráv z něj nejdříve vystupte pomocí klávesy Backspace.")
      return
    selItem = self.channelsList.StringSelection
    # some items are not channels so messages cannot be retrieved
    if (selItem == self.archiveTitle) or (selItem == self.displayAllChannelsTitle) or (selItem == self.displayAllMessagesTitle):
      return
    ui.message("načítám zprávy.")
    address = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        address = outline.get("xmlUrl")
        break
    if address:
      self.channel = RssChannel()
      self.channel.setUrl(address)
      self.channel.setChannelName(selItem)
      newArticlesCount = self.channel.readArticles()
      if newArticlesCount != -1:
        # add channel to newsChannels
        if newArticlesCount > 0:
          if selItem in self.newsChannels:
            self.newsChannels[selItem] = self.newsChannels[selItem] + newArticlesCount
          else:
            self.newsChannels[selItem] = newArticlesCount
        self.channel.writeArticles()
    if newArticlesCount <= 0:
      ui.message("Žádné nové zprávy.")
    else:
      playSound("news.wav")
      ui.message("Nových zpráv " + str(newArticlesCount))
  
  def startreadingAllNewArticles(self):
    # when only channels with new messages are displayed, it is necessary to switch to all channels
    if self.filteredNews:
      self.displayChannelsFilterItem(None)
      self.newsMenuItem.Check(False)
    # when search results are displayed, it is necessary to display all channels
    if searchFilter:
      self.clearSearch()
    newArticlesCount = 0
    speech.speakMessage("načítám zprávy.")
    progressCount = 1
    for selItem in self.channelsList.GetStrings():
      if selItem == self.archiveTitle:
        continue
      curArticlesCount = 0
      self.playProgress(progressCount, self.channelsList.GetCount())
      progressCount = progressCount + 1
      address = ""
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == selItem:
          address = outline.get("xmlUrl")
          break
      if address:
        self.channel = RssChannel()
        self.channel.setUrl(address)
        self.channel.setChannelName(selItem)
        curArticlesCount = self.channel.readArticles()
        if curArticlesCount != -1:
          newArticlesCount = newArticlesCount + curArticlesCount
          # add channel to newsChannels
          if curArticlesCount > 0:
            if selItem in self.newsChannels:
              self.newsChannels[selItem] = self.newsChannels[selItem] + curArticlesCount
            else:
              self.newsChannels[selItem] = curArticlesCount
          self.channel.writeArticles()
    if newArticlesCount == 0:
      speech.speakMessage("Žádné nové zprávy.")
    else:
      playSound("news.wav")
      speech.speakMessage("Nových zpráv " + str(newArticlesCount))
    global inProcessOfReadingNewArticles
    inProcessOfReadingNewArticles = False

  def readAllNewArticlesItem(self, evt):
    global inProcessOfReadingNewArticles
    if inProcessOfReadingNewArticles:
      speech.speakMessage("Počkejte, prosím, až bude dokončeno právě probíhající načítání nových zpráv.")
      return
    if self.inArchive:
      ui.message("Jste v archivu. Pro příjem nových zpráv z něj nejdříve vystupte pomocí klávesy Backspace.")
      return
    global searchFilter
    if not self.IsEnabled():
      # when articles are displayed and timer occurs, it is necessary to save actual state, show channel window and destroy articles window
      # partially copy of backtochannel method which is not possible to call directly from here
      if self.inArchive:
        self.channel.writeArticles(True)
      else:
        self.channel.writeArticles()
      # delete information about new messages
      selItem = self.channelsList.StringSelection
      if selItem in self.newsChannels:
        del self.newsChannels[selItem]
      self.Enable()
      self.Show()
      # when archive channel has no articles we have to delete it
      if self.inArchive:
        if self.channel.getNumberOfArticles() == 0:
          self.reallyDeleteChannel()
      # destroy articles window
      childList = self.GetChildren()
      for child in childList:
        typestr = str(type(child))
        if "ArticlesWindow" in typestr:
          child.Destroy()
          break
    # read all new articles in separate thread, without blocking main program
    inProcessOfReadingNewArticles = True
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.readAllNewArticlesItem.__qualname__}",
      target=self.startreadingAllNewArticles
    )
    t.setDaemon(True)
    t.start()
      
  def countArticles(self, chname):
    channel = RssChannel()
    channel.setChannelName(chname)
    fType = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == chname:
        fType = outline.get("type")
        break
    channel.setFeedType(fType)
    if self.inArchive:
      channel.loadArticlesFromFile(True)
    else:
      channel.loadArticlesFromFile()
    artNumber = channel.getNumberOfArticles()
    artCount = 0
    for x in range(artNumber):
      if channel.getArticleDeleted(x) == "0":
        artCount = artCount + 1
    return artCount

  def sayArticlesNumberItem(self, evt):
    sayMessage = ""
    selItem = self.channelsList.StringSelection
    # some items are not channels so number of messages cannot be retrieved
    if (selItem == self.archiveTitle) or (selItem == self.displayAllChannelsTitle):
      return
    # find number of all messages in all channels
    if selItem == self.displayAllMessagesTitle:
      artCount = 0
      for file in os.listdir(articlesFolder):
        chfile = os.path.splitext(file)[0]
        artCount = artCount + self.countArticles(chfile)
    else:
      # find number of new messages
      newMessages = 0
      if not self.inArchive:
        if selItem in self.newsChannels:
          newMessages = self.newsChannels[selItem]
          sayMessage = "Nových zpráv %s, " % (str(newMessages))
      # find number of all messages
      artCount = self.countArticles(selItem)
    sayMessage = sayMessage + "Počet zpráv %s" % (str(artCount))
    ui.message(sayMessage)

  def onchannelsListChange(self, evt):
    global currentChannelNewMessages
    global currentChannelResults
    global searchFilter
    if self.inArchive:
      return
    currentChannelNewMessages = ""
    currentChannelResults = ""
    selItem = self.channelsList.StringSelection
    if selItem in self.newsChannels:
      currentChannelNewMessages = str(self.newsChannels[selItem])
    if selItem in self.searchResults:
      currentChannelResults = str(len(self.searchResults[selItem]))

  def clearSearch(self):
    global searchFilter
    searchFilter = False
    self.searchResults.clear()
    # obtain channels from OPML
    self.choices = [outline.get("title") for outline in self.opmlTree.getroot().iter("outline")]
    self.choices.sort(key=locale.strxfrm)
    # assign channels to list items
    self.channelsList.Clear()
    for choice in self.choices:
      self.channelsList.Append(choice)
    self.addArchiveToList()
    self.channelsList.Selection = 0
    self.onchannelsListChange(None)
    self.channelsList.SetFocus()

  def displayAllArticles(self):
    global inAllArticles
    # start playing clock sound
    playSound("Clock.wav", True)
    self.channel = RssChannel()
    # read articles from all channels
    for file in os.listdir(articlesFolder):
      chfile = os.path.splitext(file)[0]
      self.channel.setChannelName(chfile)
      # find channel type
      for outline in self.opmlTree.getroot().iter("outline"):
        if outline.get("title") == chfile:
          fType = outline.get("type")
          break
      self.channel.setFeedType(fType)
      self.channel.loadArticlesFromFile()
    self.channel.setChannelName("Všechny kanály")
    # delete number of new messages in all channels
    self.newsChannels.clear()
    # sort articles by date
    self.channel.articles.sort(key=self.channel.getArticleTimestamp, reverse=True)
    inAllArticles = True
    self.Hide()
    self.Disable()
    try:
      ArticlesWindow(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def onchannelsListChoice(self, evt):
    global searchFilter
    selItem = self.channelsList.StringSelection
    if (selItem == self.displayAllMessagesTitle) and (self.channelsList.Selection == 0):
      self.displayAllArticles()
    if (selItem == self.displayAllChannelsTitle) and (searchFilter):
      self.clearSearch()
      return
    if selItem == self.archiveTitle:
      self.openArchiveItem(None)
      return
    if self.inArchive:
      self.channel = RssChannel()
      self.channel.setChannelName(selItem)
      # find feed type in filename
      for filename in os.listdir(archiveFolder):
        if filename.startswith(selItem + " ("):
          break
      fType = filename.rsplit("(", 1)[1]
      fType = fType.split(")")[0]
      self.channel.setFeedType(fType)
      self.channel.loadArticlesFromFile(True)
      self.Hide()
      self.Disable()
      try:
        ArticlesWindow(self).Show()
      except Exception as e:
        self.Enable()
        raise e
      return
    fType = ""
    address = ""
    for outline in self.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        address = outline.get("xmlUrl")
        fType = outline.get("type")
        break
    if address:
      self.channel = RssChannel()
      self.channel.setUrl(address)
      self.channel.setChannelName(selItem)
      self.channel.setFeedType(fType)
      # when automatic downloading of new articles after opening channel is on and search filter is off
      if newArticlesAfterOpening and not searchFilter:
        newArticlesCount = self.channel.readArticles()
        # when error occurs during reading new articles from Internet
        if newArticlesCount == -1:
          # behaviour is the same as reading new articles after opening channel is off
          self.channel.loadArticlesFromFile()
        else:
          # otherwise write new articles to the file
          self.channel.writeArticles()
          if newArticlesCount > 0:
            # play sound of new articles
            playSound("news.wav")
      else:
        # when automatic downloading of new articles after opening channel is off
        self.channel.loadArticlesFromFile()
      self.Hide()
      self.Disable()
      try:
        ArticlesWindow(self).Show()
      except Exception as e:
        self.Enable()
        raise e
        
  def onchannelsListKeyUp(self, evt):
    key = evt.GetKeyCode()
    evt.Skip()

  def onClose(self, evt):
    self.Destroy()
    RssReader._instance = None


class RssChannel(object):
  
  def __init__(self):
    super(RssChannel, self).__init__()
    self.channelName = ""
    self.feedType = ""
    self.url = ""
    self.numberofNewArticles = 0
    self.channelTree = None
    self.articles = []
    self.newArticles = []
    self.ns = ""
  
  def setUrl(self, url):
    self.url = url
  
  def applyNs(self, tag, ns=None):
    if ns is None:
      return tag
    else:
      return f"{ns}{tag}"
  
  def getArticleTimestamp(self, article):
    locale.setlocale(locale.LC_TIME, "en")
    try:
      if self.getFeedType() == "rss":
        date = article.find(self.applyNs("pubDate", self.ns)).text
        timestamp = time.mktime(datetime.datetime.strptime(date, "%a, %d %b %Y %H:%M:%S %z").timetuple())
      elif self.getFeedType() == "atom":
        date = article.find(self.applyNs("updated", self.ns)).text
        timestamp = time.mktime(datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%z").timetuple())
      return timestamp
    except Exception:
      try:
        if self.getFeedType() == "rss":
          date = article.find(self.applyNs("pubDate", self.ns)).text
          timestamp = time.mktime(datetime.datetime.strptime(date, "%a, %d %b %Y %H:%M:%S %Z").timetuple())
        elif self.getFeedType() == "atom":
          date = article.find(self.applyNs("updated", self.ns)).text
          timestamp = time.mktime(datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%S%Z").timetuple())
        return timestamp
      except Exception:
        return 0

  def removeDeletedArticles(self):
    # delete articles which have been deleted and are not already broadcasted
    articlesNumber = self.getNumberOfArticles()
    index = 0
    while index < articlesNumber:
      if self.articles[index].find(self.applyNs("deleted", self.ns)).text == "1":
        arTitle = self.getArticleTitle(index)
        arDate = self.getArticleDate(index)
        deletedFound = False
        for x in range(len(self.newArticles)):
          newArDate = self.getNewArticleDate(x)
          newArTitle = self.getNewArticleTitle(x)
          if arDate == newArDate and arTitle == newArTitle:
            deletedFound = True
            break
        if not deletedFound:
          del self.articles[index]
          articlesNumber = articlesNumber - 1
          index = index -1
      index = index + 1
  
  def joinArticleSources(self):
    # only new articles are added
    self.numberofNewArticles = 0
    for x in range(len(self.newArticles)):
      articleFound = False
      arDate = self.getNewArticleDate(x)
      arTitle = self.getNewArticleTitle(x)
      for y in range(self.getNumberOfArticles()):
        fiDate = self.getArticleDate(y)
        fiTitle = self.getArticleTitle(y)
        if arDate == fiDate and arTitle == fiTitle:
          articleFound = True
          break
      if not articleFound:
        addArticle = self.newArticles[x]
        # add deleted field
        delElem = ET.Element(self.applyNs("deleted", self.ns))
        delElem.text = "0"
        addArticle.append(delElem)
        self.articles.append(addArticle)
        self.numberofNewArticles += 1

  def loadArticlesFromFile(self, fromArchive=False, daysspan=0):
    channelName = self.getChannelName()
    feedType = self.getFeedType()
    if fromArchive:
      csvFileName = channelName + " (" + feedType + ")" + ".txt"
      articlesFile = os.path.join(archiveFolder, csvFileName)
    else:
      csvFileName = channelName + ".txt"
      articlesFile = os.path.join(articlesFolder, csvFileName)
    if not os.path.isfile(articlesFile):
      return False
    with open(articlesFile, 'r', encoding="utf8", newline='') as f:
      reader = csv.DictReader(f)
      for row in reader:
        if feedType == "atom":
          artElem = ET.Element(self.applyNs("entry", self.ns))
        else:
          artElem = ET.Element(self.applyNs("item", self.ns))
        title = ET.Element(self.applyNs("title", self.ns))
        title.text = row['title']
        artElem.append(title)
        if feedType == "atom":
          desc = ET.Element(self.applyNs("summary", self.ns))
        else:
          desc = ET.Element(self.applyNs("description", self.ns))
        desc.text = row['description']
        artElem.append(desc)
        if feedType == "atom":
          adate = ET.Element(self.applyNs("updated", self.ns))
        else:
          adate = ET.Element(self.applyNs("pubDate", self.ns))
        adate.text = row['date']
        artElem.append(adate)
        link = ET.Element(self.applyNs("link", self.ns))
        if feedType == "atom":
          link.set("href", row['link'])
        else:
          link.text = row['link']
        artElem.append(link)
        deleted = ET.Element(self.applyNs("deleted", self.ns))
        deleted.text = row['deleted']
        artElem.append(deleted)
        channel = ET.Element(self.applyNs("channel", self.ns))
        channel.text = channelName
        artElem.append(channel)
        feed = ET.Element(self.applyNs("feedtype", self.ns))
        feed.text = feedType
        artElem.append(feed)
        if daysspan == 0:
          self.articles.append(artElem)
        else:
          articleTimestamp = self.getArticleTimestamp(artElem)
          articleDatetime = datetime.datetime.fromtimestamp(articleTimestamp)
          currentDatetime = datetime.datetime.now()
          diffDatetime = currentDatetime - articleDatetime
          diffDays = diffDatetime.days
          if diffDays <= daysspan:
            self.articles.append(artElem)
    return True

  def readArticles(self):
    try:
      response = urllib.request.urlopen(self.url)
    except Exception:
      userAgent = "UniversalFeedParser/5.0.1 +http://feedparser.org/"
      headers = {'User-Agent': userAgent}
      req = urllib.request.Request(self.url, None, headers)
      try:
        response = urllib.request.urlopen(req)
      except Exception as e:
        playSound("error.wav")
        speech.speakMessage("Chyba při načítání kanálu %s: %s" % (self.channelName, e))
        return -1
    try:
      xmlstr = response.read().strip()
      self.channelTree = ET.fromstring(xmlstr)
    except Exception as e:
      playSound("error.wav")
      speech.speakMessage("Chyba při načítání kanálu %s: %s" % (self.channelName, e))
      return -1
    tag = self.channelTree.tag
    self.ns = "%s}" % tag.split("}", 1)[0] if "}" in tag else None
    # rss or atom
    if tag.endswith("rss"):
      self.main = self.channelTree.find(self.applyNs("channel", self.ns))
      self.newArticles = self.main.findall(self.applyNs("item", self.ns))
      self.feedType = "rss"
    elif tag.endswith("feed"):
      self.main = self.channelTree
      self.newArticles = self.main.findall(self.applyNs("entry", self.ns))
      self.feedType = "atom"
    else:
      log.debugWarning("Neznámý typ RSS kanálu", exc_info=True)
      raise
    # remove HTML tags from title and description of all new articles
    for article in self.newArticles:
      # remove tags from title
      try:
        arTitleTr = article.find(self.applyNs("title", self.ns))
        soup = BeautifulSoup(arTitleTr.text)
        arTitleTr.text = soup.getText(strip=True)
      except: pass
      # remove tags from description
      try:
        if self.getFeedType() == "rss":
          arDescTr = article.find(self.applyNs("description", self.ns))
        elif self.getFeedType() == "atom":
          arDescTr = article.find(self.applyNs("summary", self.ns))
        soup = BeautifulSoup(arDescTr.text)
        arDescTr.text = soup.getText(strip=True)
      except: pass
    self.setChannelName(validFileName(self.getChannelName()))
    if self.loadArticlesFromFile():
      self.removeDeletedArticles()
    self.joinArticleSources()
    self.articles.sort(key=self.getArticleTimestamp, reverse=True)
    self.index = 0
    return self.numberofNewArticles

  def writeArticles(self, fromArchive=False):
    global articlesHeader
    channelName = self.getChannelName()
    feedType = self.getFeedType()
    if fromArchive:
      csvFileName = channelName + " (" + feedType + ")" + ".txt"
      articlesFile = os.path.join(archiveFolder, csvFileName)
    else:
      csvFileName = channelName + ".txt"
      articlesFile = os.path.join(articlesFolder, csvFileName)
    with open(articlesFile, 'w', encoding="utf8", newline='') as f:
      writer = csv.writer(f)
      # write the header
      writer.writerow(articlesHeader)
      # write articles
      for x in range(self.getNumberOfArticles()):
        arTitle = self.getArticleTitle(x)
        arDesc = self.getArticleDescription(x)
        arDate = self.getArticleDate(x)
        arLink = self.getArticleLink(x)
        arDeleted = self.getArticleDeleted(x)
        articleRow = [arTitle, arDesc, arDate, arLink, arDeleted]
        writer.writerow(articleRow)

  def getChannelUrl(self):
    return self.url

  def getFeedType(self):
    return self.feedType

  def setFeedType(self, fType=""):
    self.feedType = fType

  def setChannelName(self, chName=""):
    self.channelName = chName

  def getChannelName(self):
    if self.channelName:
      return self.channelName
    else:
      try:
        self.setChannelName(self.main.find(self.applyNs("title", self.ns)).text.strip())
        return self.channelName
      except Exception:
        return ""

  def getChannelNamefromArticle(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("channel", self.ns)).text
    except:
      return ""

  def getFeedTypefromArticle(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("feedtype", self.ns)).text
    except:
      return ""

  def getArticleTitle(self, index=None):
    if index is None:
      index = self.index
    try:
      arTitle = self.articles[index].find(self.applyNs("title", self.ns)).text
    except Exception:
      arTitle = "Nelze získat titulek zprávy."
    if not arTitle:
      arTitle = "Bez titulku"
    return arTitle

  def getNewArticleTitle(self, index=None):
    if index is None:
      index = self.index
    try:
      arTitle = self.newArticles[index].find(self.applyNs("title", self.ns)).text
    except Exception:
      arTitle = "Nelze získat titulek zprávy."
    if not arTitle:
      arTitle = "Bez titulku"
    return arTitle

  def getArticleLink(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        return self.articles[index].find(self.applyNs("link", self.ns)).text
      elif self.getFeedType() == "atom":
        return self.articles[index].find(self.applyNs("link", self.ns)).get("href")
    except Exception:
      return "Nelze získat odkaz na zprávu."

  def getNewArticleLink(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        return self.newArticles[index].find(self.applyNs("link", self.ns)).text
      elif self.getFeedType() == "atom":
        return self.newArticles[index].find(self.applyNs("link", self.ns)).get("href")
    except Exception:
      return "Nelze získat odkaz na zprávu."

  def getArticleDeleted(self, index=None):
    if index is None:
      index = self.index
    try:
      return self.articles[index].find(self.applyNs("deleted", self.ns)).text
    except Exception:
      return None

  def getArticleDescription(self, index=None):
    if index is None:
      index = self.index
    description = None
    try:
      if self.getChannelUrl().startswith("https://www.google.com") and "/alerts/" in self.getChannelUrl():
        description = self.articles[index].find(self.applyNs("content", self.ns)).text
      if description is not None:
        return description
      if self.getFeedType() == "rss":
        description = self.articles[index].find(self.applyNs("description", self.ns)).text
      elif self.getFeedType() == "atom":
        description = self.articles[index].find(self.applyNs("summary", self.ns)).text
      if description is None:
        description = self.articles[index].find(self.applyNs("content", self.ns)).text
      return description
    except Exception:
      return None

  def getNewArticleDate(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        date = self.newArticles[index].find(self.applyNs("pubDate", self.ns)).text
      elif self.getFeedType() == "atom":
        date = self.newArticles[index].find(self.applyNs("updated", self.ns)).text
      return date
    except Exception:
      return ""

  def getArticleDate(self, index=None):
    if index is None:
      index = self.index
    try:
      if self.getFeedType() == "rss":
        date = self.articles[index].find(self.applyNs("pubDate", self.ns)).text
      elif self.getFeedType() == "atom":
        date = self.articles[index].find(self.applyNs("updated", self.ns)).text
      return date
    except Exception:
      return ""

  def getNumberOfArticles(self):
    return len(self.articles)

class ArticlesWindow(wx.Frame):

  needSave = False
  deleteAllAfter = defaultdict(list)

  def __init__(self, parent):
    # The window title
    super(ArticlesWindow, self).__init__(parent, size=(1000, 800), title="Zprávy - " + parent.channel.getChannelName())
    if not inAllArticles:
      newMessagesCount = 0
      newMessagesCount = parent.channel.numberofNewArticles
      if parent.channel.getChannelName() in parent.newsChannels:
        newMessagesCount = newMessagesCount + parent.newsChannels[parent.channel.getChannelName()]
      if newMessagesCount > 0:
        self.SetTitle("Zprávy - " + parent.channel.getChannelName() + " (" + str(newMessagesCount) + ")")

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    articlesChoices = []

    # the menu bar
    menuBar = wx.MenuBar()
    menu = wx.Menu()
    
    # menu items
    item = menu.Append(wx.ID_ANY, "Zobrazit &celou zprávu \tEnter")
    self.Bind(wx.EVT_MENU, self.openArticleItem, item)
    item = menu.Append(wx.ID_ANY, "Zobrazit celou zprávu v &prohlížeči\tCtrl+Enter")
    self.Bind(wx.EVT_MENU, self.openArticleInBrowserItem, item)
    item = menu.Append(wx.ID_ANY, "Zobrazit celou zprávu jako &text\tShift+Enter")
    self.Bind(wx.EVT_MENU, self.openArticleAsTextItem, item)
    item = menu.Append(wx.ID_ANY, "&Smazat zprávu\tDelete")
    self.Bind(wx.EVT_MENU, self.DeleteArticleItem, item)
    item = menu.Append(wx.ID_ANY, "&Ohlásit popis zprávy\tCtrl+F2")
    self.Bind(wx.EVT_MENU, self.sayArticleDescriptionItem, item)
    item = menu.Append(wx.ID_ANY, "&Ohlásit datum a pozici zprávy\tF2")
    self.Bind(wx.EVT_MENU, self.sayArticleDateItem, item)
    item = menu.Append(wx.ID_ANY, "Poslat zprávu &mailem\tCtrl+E")
    self.Bind(wx.EVT_MENU, self.sendArticleItem, item)
    item = menu.Append(wx.ID_ANY, "&Archivovat zprávu\tF6")
    self.Bind(wx.EVT_MENU, self.archiveArticleItem, item)
    item = menu.Append(wx.ID_ANY, "Zpět na &seznam kanálů\tBackspace")
    self.Bind(wx.EVT_MENU, self.backtoChannelsItem, item)
    menu.AppendSeparator()
    item = menu.Append(wx.ID_EXIT, "U&končit")
    self.Bind(wx.EVT_MENU, self.onClose, item)
    
    # the menu title
    menuBar.Append(menu, "Zprávy")
    self.SetMenuBar(menuBar)
    
    # obtain articles
    for index in range(parent.channel.getNumberOfArticles()):
      articleDeleted = parent.channel.getArticleDeleted(index)
      # display only articles which were not deleted
      if articleDeleted == "0":
        articleTitle = parent.channel.getArticleTitle(index)
        if not articleTitle:
          articleTitle = "Nelze získat titulek zprávy."
        articlesChoices.append(articleTitle)

    # filter articles if search is active
    global searchFilter
    if searchFilter:    
      searchResultChoices = parent.searchResults[parent.channel.getChannelName()]
      articlesChoices = list(set(articlesChoices) & set(searchResultChoices))
    # assign articles to listbox
    self.articlesList = wx.ListBox(self, choices=articlesChoices)
    self.articlesList.Bind(wx.EVT_LISTBOX, self.onarticlesListChange)
    self.articlesList.Bind(wx.EVT_LISTBOX_DCLICK, self.onarticlesListChoice)
    self.articlesList.Bind(wx.EVT_KEY_UP, self.onarticlesListKeyUp)
    self.articlesList.SetMinSize((1000,800));
    sHelper.addItem(self.articlesList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    # stop sound if all articles from all channels are displayed
    if inAllArticles:
      stopsound()
    
    self.articlesList.SetFocus()
    self.CentreOnScreen()
    if self.articlesList.GetCount() > 0:
      self.articlesList.Selection = 0
      self.onarticlesListChange(None)
    else:
      self.noArticles()

  def getSelectedArticleIndex(self):
      for index in range(self.Parent.channel.getNumberOfArticles()):
        articleTitle = self.Parent.channel.getArticleTitle(index)
        if articleTitle == self.articlesList.StringSelection:
          break
      return index

  def noArticles(self):
    dlgMessage = "V tomto kanálu nejsou žádné zprávy."
    dlgTitle = "Informace"
    wx.CallAfter(gui.messageBox, dlgMessage, dlgTitle, wx.OK)
    wx.CallAfter(self.backtoChannelsItem, None)

  def DeleteArticleItem(self, evt, fromArchive=False):
    global inAllArticles
    if self.articlesList.GetCount() == 0:
      return
    delItem = self.articlesList.StringSelection
    if askArticleDeletion and not fromArchive:
      dlgMessage = "Opravdu chcete odstranit zprávu " + delItem + "?"
      dlgTitle = "Upozornění"
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.YES != result:
        return
    index = self.getSelectedArticleIndex()
    # when all articles is displayed
    if inAllArticles:
      # find channel name and article title from currently selected article
      delChannelNamelm = self.Parent.channel.articles[index].find(self.Parent.channel.applyNs("channel", self.Parent.channel.ns))
      delChannelName = delChannelNamelm.text
      delTitlelm = self.Parent.channel.articles[index].find(self.Parent.channel.applyNs("title", self.Parent.channel.ns))
      delTitle = delTitlelm.text
      # assign channel name as key to dict and add article title to list, value of this dict
      self.deleteAllAfter[delChannelName].append(delTitle)
    else:
      # when articles from one channel are displayed
      if self.Parent.inArchive:
        del self.Parent.channel.articles[index]
      else:
        delMark = self.Parent.channel.articles[index].find(self.Parent.channel.applyNs("deleted", self.Parent.channel.ns))
        delMark.text = "1"
    self.needSave = True
    selItemNumber = self.articlesList.GetSelection()
    if selItemNumber != -1:
      self.articlesList.Delete(selItemNumber)
      # delete article from search results list in searchresult dictionary
      if searchFilter:
        self.Parent.searchResults[self.Parent.channel.getChannelName()].remove(delItem)
      if self.articlesList.GetCount() > 0:
        if selItemNumber == self.articlesList.GetCount():
          self.articlesList.Selection = selItemNumber-1
        else:
          self.articlesList.Selection = selItemNumber
        self.onarticlesListChange(None)
      else:
        self.noArticles()

  def archiveArticleItem(self, evt):
    if askArticleArchivation:
      dlgMessage = "Chcete přesunout do archivu zprávu " + self.articlesList.StringSelection + "?"
      dlgTitle = "Upozornění"
      result = gui.messageBox(
        message=dlgMessage,
        caption=dlgTitle,
        style=wx.YES | wx.NO | wx.ICON_WARNING
      )
      if wx.YES != result:
        return
    global articlesHeader
    if not os.path.exists(archiveFolder):
      os.mkdir(archiveFolder)
    index = self.getSelectedArticleIndex()
    articleTitle = self.Parent.channel.getArticleTitle(index)
    articleDesc = self.Parent.channel.getArticleDescription(index)
    articleDate = self.Parent.channel.getArticleDate(index)
    articleLink = self.Parent.channel.getArticleLink(index)
    if inAllArticles:
      channelName = self.Parent.channel.getChannelNamefromArticle(index)
      feedType = self.Parent.channel.getFeedTypefromArticle(index)
    else:
      channelName = self.Parent.channel.getChannelName()
      feedType = self.Parent.channel.getFeedType()
    csvFileName = channelName + " (" + feedType + ")" + ".txt"
    archiveFile = os.path.join(archiveFolder, csvFileName)
    writeheader = False
    if not os.path.isfile(archiveFile):
      writeheader = True
    with open(archiveFile, 'a', encoding="utf8", newline='') as f:
      writer = csv.writer(f)
      if writeheader:
        # write the header
        writer.writerow(articlesHeader)
      # write article
      articleRow = [articleTitle, articleDesc, articleDate, articleLink, "0"]
      writer.writerow(articleRow)
    self.DeleteArticleItem(None, True)
    if self.Parent.channelsList.GetString(self.Parent.channelsList.GetCount()-1) != self.Parent.archiveTitle:
      self.Parent.channelsList.InsertItems([self.Parent.archiveTitle], self.Parent.channelsList.GetCount())

  def sendArticleItem(self, evt):
    index = self.getSelectedArticleIndex()
    subject = self.Parent.channel.getArticleTitle(index)
    body = self.Parent.channel.getArticleDescription(index)
    body = body + "\n"
    body = body + "Zdroj: " + self.Parent.channel.getArticleLink(index)
    webbrowser.open('mailto:?subject=' + subject + '&body=' + body, new=1)

  def sayArticleDateItem(self, evt):
    locale.setlocale(locale.LC_TIME, "en")
    artDateTime = None
    sayMessage = ""
    index = self.getSelectedArticleIndex()
    articleDate = self.Parent.channel.getArticleDate(index)
    if articleDate:
      try:
        artDateTime = datetime.datetime.strptime(articleDate, "%a, %d %b %Y %H:%M:%S %z")
      except Exception:
        try:
          # another date and time format
          artDateTime = datetime.datetime.strptime(articleDate, "%a, %d %b %Y %H:%M:%S %Z")
        except Exception:
          try:
            # date and time format in atom
            artDateTime = datetime.datetime.strptime(articleDate, "%Y-%m-%dT%H:%M:%S%z")
          except Exception:
            artDateTime = None
      if artDateTime:
        locale.setlocale(locale.LC_TIME, "czech")
        datestr = datetime.datetime.strftime(artDateTime, "%d. %B %Y %H:%M:%S")
        sayMessage = datestr
      else:
        sayMessage = "Datum nelze získat."
    if sayMessage:
      sayMessage = sayMessage + ", "
    sayMessage = sayMessage + "Zpráva %s z %s" % (str(self.articlesList.GetSelection() + 1), str(self.articlesList.GetCount()))
    if inAllArticles:
      sayMessage = self.Parent.channel.getChannelNamefromArticle(index) + ", " + sayMessage
    ui.message(sayMessage)

  def sayArticleDescriptionItem(self, evt):
    index = self.getSelectedArticleIndex()
    articleDesc = self.Parent.channel.getArticleDescription(index)
    if articleDesc:
      ui.message(articleDesc)
    else:
      ui.message("Žádný popis.")
        
  def openArticleItem(self, evt):
    if openArticlesAsText:
      self.openArticleAsTextItem(None)
    else:
      self.openArticleInBrowserItem(None)

  def openArticleInBrowserItem(self, evt):
    index = self.getSelectedArticleIndex()
    articleAddress = self.Parent.channel.getArticleLink(index)
    os.startfile(articleAddress)

  def displayText(self):
    self.Hide()
    self.Disable()
    try:
      TextWindow(self).Show()
    except Exception as e:
      self.Enable()
      raise e

  def openArticleAsTextItem(self, evt):
    index = self.getSelectedArticleIndex()
    articleAddress = self.Parent.channel.getArticleLink(index)
    if "idnes.cz" in articleAddress:
      req = requests.get(articleAddress, cookies={'dCMP': '"mafra=1111,all=1,reklama=1,part=0,cpex=1,google=1,gemius=1,id5=1,next=0000,onlajny=0000,jenzeny=0000,databazeknih=0000,autojournal=0000,skodahome=0000,skodaklasik=0000,groupm=1,piano=1,seznam=1,geozo=0,czaid=1,click=1,verze=2,'})
    else:
      req = requests.get(articleAddress)
    pageHtml = req.text
    # find article starting point
    # search for h1
    headingPos = pageHtml.find("<h1")
    # if not found, search for <main
    if headingPos == -1:
      headingPos = pageHtml.find("<main")
    # when nothing is found, set starting pos to 0
    if headingPos == -1:
      headingPos = 0
    # find article end
    # search for </article> (from the end)
    endPos = pageHtml.rfind("</article>", headingPos)
    # if not found, search for next landmark
    if endPos == -1:
      endPos = pageHtml.find("<aside", headingPos)
    if endPos == -1:
      endPos = pageHtml.find("<nav", headingPos)
    if endPos == -1:
      endPos = pageHtml.find("<footer", headingPos)
    # if nothing is found, set end to end of text
    if endPos == -1:
      endPos = len(pageHtml)
    pageEditHtml = pageHtml[headingPos:endPos]

    soup = BeautifulSoup(pageEditHtml)
    pageText = soup.getText(separator="\n", strip=True)
    TextWindow.dlgTitle = "Textový prohlížeč"
    TextWindow.dispText = pageText
    self.displayText()

  def saveArticles(self):
    global inAllArticles
    if inAllArticles:
      # mark all deleted articles in all needed channels
      global articlesHeader
      for channelName, articles in self.deleteAllAfter.items():
        csvFileName = channelName + ".txt"
        articlesFile = os.path.join(articlesFolder, csvFileName)
        articlesFileOut = os.path.join(articlesFolder, csvFileName+"tmp")
        # open file with articles and temp file for storing copy with marked articles for deletion
        with open(articlesFile, 'r', encoding="utf8", newline='') as f, open(articlesFileOut, 'w', encoding="utf8", newline='') as fout:
          reader = csv.DictReader(f)
          writer = csv.writer(fout)
          # write the header
          writer.writerow(articlesHeader)
          for row in reader:
            if row['title'] in articles:
              row['deleted'] = "1"
            rowout = [row['title'], row['description'], row['date'], row['link'], row['deleted']]
            writer.writerow(rowout)
        if os.path.isfile(articlesFileOut):
          os.remove(articlesFile)
          os.rename(articlesFileOut, articlesFile)
      self.deleteAllAfter.clear()
    else:
      if self.Parent.inArchive:
        self.Parent.channel.writeArticles(True)
      else:
        self.Parent.channel.writeArticles()
    self.needSave = False

  def backtoChannelsItem(self, item):
    global inAllArticles
    if inAllArticles:
      if self.needSave:
        self.saveArticles()
      inAllArticles = False
    else:
      if self.needSave:
        self.saveArticles()
      # delete information about new messages
      selItem = self.Parent.channelsList.StringSelection
      if selItem in self.Parent.newsChannels:
        del self.Parent.newsChannels[selItem]
        if self.Parent.filteredNews:
          if len(self.Parent.newsChannels) > 0:
            delItem = self.Parent.channelsList.GetSelection()
            cursel = self.Parent.channelsList.GetSelection()
            self.Parent.channelsList.Delete(delItem)
            if self.Parent.channelsList.GetCount() == cursel:
              cursel = cursel -1
            self.Parent.channelsList.Selection = cursel
          else:
            self.Parent.newsMenuItem.Check(False)
            self.Parent.displayChannelsFilterItem(None)
        self.Parent.onchannelsListChange(None)
      # when search results are displayed, refresh of additional info is needed
      if searchFilter:
        self.Parent.onchannelsListChange(None)
    self.Parent.Enable()
    self.Parent.Show()
    # when archive channel has no articles we have to delete it
    if self.Parent.inArchive:
      if self.Parent.channel.getNumberOfArticles() == 0:
        self.Parent.reallyDeleteChannel()
    self.Destroy()

  def onarticlesListChange(self, evt):
    global currentArticleDescription
    index = self.getSelectedArticleIndex()
    articleDesc = self.Parent.channel.getArticleDescription(index)
    if articleDesc:
      currentArticleDescription = articleDesc
    else:
      currentArticleDescription = ""

  def onarticlesListChoice(self, evt):
    self.openArticleItem(None)

  def onarticlesListKeyUp(self, evt):
    key = evt.GetKeyCode()
    if key == wx.WXK_ESCAPE:
      self.backtoChannelsItem(None)
    evt.Skip()

  def onClose(self, evt):
    global inAllArticles
    if inAllArticles:
      if self.needSave:
        self.saveArticles()
      inAllArticles = False
    else:
      if self.needSave:
        self.saveArticles()
    self.Parent.Destroy()
    self.Destroy()
    RssReader._instance = None


class ChannelPropertiesDialog(wx.Dialog):

  chName = ""
  chAddress = ""
  
  def __init__(self, parent):
    
    # The title of a dialog.
    super(ChannelPropertiesDialog, self).__init__(parent, title="Vlastnosti kanálu")
    self.Bind(wx.EVT_CLOSE, self.onClose)

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The label of a channel name text field
    channelNameLabel = "&Název kanálu:"
    self.channelNameEdit = sHelper.addLabeledControl(channelNameLabel, wx.TextCtrl)

    # The label of a channel address text field
    channelAddressLabel = "&Adresa kanálu:"
    self.channelAddressEdit = sHelper.addLabeledControl(channelAddressLabel, wx.TextCtrl)
  
    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label=_(u"&Zrušit"))
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    # find channel name and address
    selItem = parent.channelsList.StringSelection
    self.chName = ""
    self.chAddress = ""
    for outline in parent.opmlTree.getroot().iter("outline"):
      if outline.get("title") == selItem:
        self.chName = selItem
        self.chAddress = outline.get("xmlUrl")
        break
    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.channelNameEdit.SetValue(self.chName)
    self.channelAddressEdit.SetValue(self.chAddress)
    self.channelNameEdit.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    if self.channelNameEdit == "" or self.channelAddressEdit == "":
      self.EndModal(False)
      self.onClose(self)
      return
    csvFileName = self.chName + ".txt"
    articlesFile = os.path.join(articlesFolder, csvFileName)
    # write changes to OPML file
    for outline in self.Parent.opmlTree.getroot().iter("outline"):
      if outline.get("title") == self.chName:
        self.chName = validFileName(self.channelNameEdit.Value)
        self.chAddress = self.channelAddressEdit.Value
        self.Parent.newChannelName = self.chName
        outline.set("title", self.chName)
        outline.set("text", self.chName)
        outline.set("xmlUrl", self.chAddress)
        outline.set("type", "rss")
        with open(opmlFilePath, 'wb') as f:
          f.write(ET.tostring(self.Parent.opmlTree.getroot(), encoding='utf-8', method='xml'))
    # rename file with articles
    if os.path.isfile(articlesFile):
      newCsvFileName = self.chName + ".txt"
      newArticlesFile = os.path.join(articlesFolder, newCsvFileName)
      if articlesFile != newArticlesFile:
        os.rename(articlesFile, newArticlesFile)
    self.EndModal(True)
    self.onClose(self)

  def onCancel(self, evt):
    self.EndModal(False)
    self.onClose(self)

  def onClose(self, evt):
    self.Destroy()


class ChannelDatabaseDialog(wx.Dialog):
  
  channelsDisplayed = False
  channelsDict = dict()
  selCat = ""
  selChan = ""
  passkey = False

  def __init__(self, parent):
    # The title of a dialog
    super(ChannelDatabaseDialog, self).__init__(parent, title="Databáze RSS kanálů")

    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)
    self.Bind(wx.EVT_CLOSE, self.onClose)

    # get RSS categories from XML
    xmlUrl = "http://suscik.cz/nvda/rss/feeds.xml"
    offlinedb = False
    try:
      response = urlopen(xmlUrl).read()
    except:
      offlinedb = True
    if not offlinedb:
      self.tree = ET.fromstring(response)
    else:
      xmlFile = os.path.join(os.path.dirname(__file__), "files", "feeds.xml")
      treexml = ET.parse(xmlFile)
      self.tree = treexml.getroot()
      self.SetTitle("Databáze RSS kanálů (místní databáze)")

    # listbox with categories
    self.choices = []
    self.choices = [x.attrib["name"] for x in self.tree]
    self.choices.sort(key=locale.strxfrm)
    self.categoriesList = wx.ListBox(self, choices=self.choices)
    self.categoriesList.Selection = 0
    self.categoriesList.Bind(wx.EVT_LISTBOX_DCLICK, self.oncategoriesListChoice)
    self.categoriesList.Bind(wx.EVT_KEY_UP, self.oncategoriesListKeyDown)
    sHelper.addItem(self.categoriesList, flag = wx.EXPAND)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    self.Sizer = mainSizer

    self.categoriesList.SetFocus()
    self.CentreOnScreen()

  def displayCategories(self):
    # fill listbox with categories
    self.choices = [x.attrib["name"] for x in self.tree]
    self.choices.sort(key=locale.strxfrm)
    self.categoriesList.Clear()
    self.categoriesList.InsertItems(self.choices, 0)
    self.categoriesList.Selection = self.categoriesList.FindString(self.selCat)
    
  def displayChannels(self, category):
    self.channelsDict = [x.attrib for x in self.tree.find(f"Category[@name='{category}']")]
    # fill list with channels
    self.choices = [x["Description"] for x in self.channelsDict]
    self.choices.sort(key=locale.strxfrm)
    self.categoriesList.Clear()
    self.categoriesList.InsertItems(self.choices, 0)
    self.categoriesList.Selection = 0

  def findChannelAddress(self):
    for x in self.channelsDict:
      if x["Description"] == self.selChan:
        self.Parent.dbSelectedUrl = x["URL"]
        break
    self.onClose(self)

  def oncategoriesListChoice(self, evt):
    if not self.channelsDisplayed:
      self.channelsDisplayed = True
      self.selCat = self.categoriesList.StringSelection
      self.displayChannels(self.selCat)
    else:
      self.selChan = self.categoriesList.StringSelection
      self.findChannelAddress()

  def oncategoriesListKeyDown(self, evt):
    key = evt.GetKeyCode()
    if self.passkey:
      self.passkey = False
      return
    if key == wx.WXK_RETURN or key == wx.WXK_NUMPAD_ENTER:
      self.oncategoriesListChoice(self)
    elif key == wx.WXK_BACK:
      if self.channelsDisplayed:
        self.channelsDisplayed = False
        self.displayCategories()
    elif key == wx.WXK_ESCAPE:
      self.onClose(self)
    evt.Skip()

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()


class RssReaderSettingsDialog(wx.Dialog):

  def __init__(self, parent):
    
    # The title of a dialog.
    super(RssReaderSettingsDialog, self).__init__(parent, title="Nastavení pro RSS Čtečku")

    self.Bind(wx.EVT_CLOSE, self.onClose)
    mainSizer = wx.BoxSizer(wx.VERTICAL)
    sHelper = guiHelper.BoxSizerHelper(self,orientation=wx.VERTICAL)

    # The open articles as text checkbox
    self.openArticlesAsTextCheck = wx.CheckBox(self, -1, "Otevírat zprávy jako &text")
    self.openArticlesAsTextCheck.SetValue(bool(openArticlesAsText))
    sHelper.addItem(self.openArticlesAsTextCheck)

    # The ask article deletion checkbox
    self.askArticleDeletionCheck = wx.CheckBox(self, -1, "Zeptat se před smazáním &zprávy")
    self.askArticleDeletionCheck.SetValue(bool(askArticleDeletion))
    sHelper.addItem(self.askArticleDeletionCheck)

    # The ask article archivation checkbox
    self.askArticleArchivationCheck = wx.CheckBox(self, -1, "Zeptat se před přesunem zprávy do &archivu")
    self.askArticleArchivationCheck.SetValue(bool(askArticleArchivation))
    sHelper.addItem(self.askArticleArchivationCheck)

    # The automatically announce number of new messages checkbox
    self.announceNumberofNewsCheck = wx.CheckBox(self, -1, "Automaticky ohlásit p&očet nových zpráv u kanálu")
    self.announceNumberofNewsCheck.SetValue(bool(automaticallyAnnounceNumberofNews))
    sHelper.addItem(self.announceNumberofNewsCheck)

    # The automatically announce description checkbox
    self.announceDescriptionCheck = wx.CheckBox(self, -1, "Automaticky ohlásit &popis u zprávy")
    self.announceDescriptionCheck.SetValue(bool(automaticallyAnnounceDescription))
    sHelper.addItem(self.announceDescriptionCheck)

    # The new articles after opening checkbox
    self.newArticlesAfterOpeningCheck = wx.CheckBox(self, -1, "Po otevření &kanálu načíst nové zprávy")
    self.newArticlesAfterOpeningCheck.SetValue(bool(newArticlesAfterOpening))
    sHelper.addItem(self.newArticlesAfterOpeningCheck)

    # The download articles after startup checkbox
    self.downloadArticlesAfterStartupCheck = wx.CheckBox(self, -1, "Po &spuštění načíst nové zprávy")
    self.downloadArticlesAfterStartupCheck.SetValue(bool(downloadArticlesAfterStartup))
    sHelper.addItem(self.downloadArticlesAfterStartupCheck)

    # The download articles interval combobox
    intervalChoices = ['Vypnuto', '5 minut', '10 minut', '15 minut', '30 minut', '1 hodina', '2 hodiny', '3 hodiny']
    self.downloadArticlesIntervalCombo = sHelper.addLabeledControl(_("Automaticky stahovat nové zprávy"), wx.Choice, choices=intervalChoices)
    self.downloadArticlesIntervalCombo.SetSelection(downloadArticlesInterval)

    #button row
    sHelper_Buttons = guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)

    # The label of an OK button 
    self.okButton = wx.Button(self, label="&OK")
    self.okButton.Bind(wx.EVT_BUTTON, self.onOk)
    self.AffirmativeId = self.okButton.Id
    self.okButton.SetDefault()
    sHelper_Buttons.addItem(self.okButton)

    # The label of a Cancel button 
    cancelButton = wx.Button(self, wx.ID_CLOSE, label="&Zrušit")
    cancelButton.Bind(wx.EVT_BUTTON, lambda evt: self.Close())
    sHelper_Buttons.addDialogDismissButtons(cancelButton)
    self.Bind(wx.EVT_CLOSE, self.onCancel)
    self.EscapeId = wx.ID_CLOSE

    sHelper.addItem(sHelper_Buttons, flag = wx.CENTER)

    mainSizer.Add(sHelper.sizer, flag=wx.ALL, border=guiHelper.BORDER_FOR_DIALOGS)
    mainSizer.Fit(self)
    self.Sizer = mainSizer
    self.openArticlesAsTextCheck.SetFocus()
    self.CentreOnScreen()

  def onOk(self, evt):
    # Load settings
    SetFile = settingsFilePath
    with open(SetFile, 'r') as f:
      config = json.load(f)
    # Assign values to config
    config['OpenArticlesAsText'] = self.openArticlesAsTextCheck.GetValue()
    config['AskArticleDeletion'] = self.askArticleDeletionCheck.GetValue()
    config['AskArticleArchivation'] = self.askArticleArchivationCheck.GetValue()
    config['AutomaticallyAnnounceNumberofNews'] = self.announceNumberofNewsCheck.GetValue()
    config['AutomaticallyAnnounceDescription'] = self.announceDescriptionCheck.GetValue()
    config['NewArticlesAfterOpening'] = self.newArticlesAfterOpeningCheck.GetValue()
    config['DownloadArticlesAfterStartup'] = self.downloadArticlesAfterStartupCheck.GetValue()
    config['DownloadArticlesInterval'] = self.downloadArticlesIntervalCombo.GetSelection()
    # Save the settings
    with open(SetFile, 'w') as f:
      json.dump(config, f)
    # Apply settings
    global openArticlesAsText
    openArticlesAsText = config['OpenArticlesAsText']
    global askArticleDeletion
    askArticleDeletion = config['AskArticleDeletion']
    global askArticleArchivation
    askArticleArchivation = config['AskArticleArchivation']
    global automaticallyAnnounceNumberofNews 
    automaticallyAnnounceNumberofNews = config['AutomaticallyAnnounceNumberofNews']
    global automaticallyAnnounceDescription 
    automaticallyAnnounceDescription = config['AutomaticallyAnnounceDescription']
    global newArticlesAfterOpening
    newArticlesAfterOpening = config['NewArticlesAfterOpening']
    global downloadArticlesAfterStartup
    downloadArticlesAfterStartup = config['DownloadArticlesAfterStartup']
    global downloadArticlesInterval
    downloadArticlesInterval = config['DownloadArticlesInterval']
    if downloadArticlesInterval == 0:
      self.Parent.regularTimer.Stop()
    else:
      self.Parent.regularTimer.Start(convertDownloadArticlesInterval(downloadArticlesInterval))
    self.onClose(None)

  def onCancel(self, evt):
    self.onClose(None)

  def onClose(self, evt):
    self.Parent.Enable()
    self.Parent.Show()
    self.Destroy()


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

  # category for addon commands in input gestures dialog
  scriptCategory = _addonSummary

  def __init__(self, *args, **kwargs):
    super(GlobalPlugin, self).__init__(*args, **kwargs)

    self.toolsMenu = gui.mainFrame.sysTrayIcon.toolsMenu
    self.runRssItem = self.toolsMenu.Append(wx.ID_ANY, "&Rss čtečka")
    gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onRssReader, self.runRssItem)

    # check for update
    self.updateCheck()

  def updateCheck(self):
    isNewVersion = False
    #determines the version from the web
    data = ""
    versionUrl = "http://suscik.cz/nvda/rss/ver.txt"
    try:
      response = urlopen(versionUrl).read()
    except:
      response = ""
    if response:
      data = response.decode()
    if data:
      #checks if new version is available
      isNewVersion = False
      currentVersion = data
      if (int(currentVersion.split(".")[0]) > int(_addonVersion.split(".")[0])):
        isNewVersion = True
      elif (int(currentVersion.split(".")[1]) > int(_addonVersion.split(".")[1])):
        isNewVersion = True
      elif (int(currentVersion.split(".")[2]) > int(_addonVersion.split(".")[2])):
        isNewVersion = True
    if isNewVersion:
      wx.CallAfter(self.updateAddon, currentVersion)
  
  def updateAddon(self, version):
    message = '%s %s %s %s.\n%s' % (
      "Je k dispozici aktualizace", _addonSummary, "verze", version,
      "Chcete ji stáhnout?")
    if gui.messageBox(message, "Kontrola aktualizace", wx.YES | wx.NO | wx.ICON_QUESTION) == wx.YES:
      #save to downloads
      updateUrl = "http://suscik.cz/nvda/rss/rssreader.nvda-addon"
      from os.path import expanduser
      home = expanduser("~")
      target = os.path.join(home, "downloads", "rssreader.nvda-addon")
      self.downloadUrl = updateUrl
      self.downloadTarget = target
      self.startDownload()

  # downloading with progressbar
  # taken from NVDA source, slightly edited

  def startDownload(self):
    self._shouldCancel = False
    self._guiExecTimer = gui.NonReEntrantTimer(self._guiExecNotify)
    self._progressDialog = wx.ProgressDialog("Stahuji",
      "Připojuji",
      style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME | wx.PD_AUTO_HIDE,
      )
    self._progressDialog.CentreOnScreen()
    self._progressDialog.Raise()
    t = threading.Thread(
      name=f"{self.__class__.__module__}.{self.startDownload.__qualname__}",
      target=self._bg
    )
    t.daemon = True
    t.start()

  def _guiExec(self, func, *args):
    self._guiExecFunc = func
    self._guiExecArgs = args
    if not self._guiExecTimer.IsRunning():
      wx.CallAfter(self._guiExecTimer.Start, 50, True)

  def _guiExecNotify(self):
    self._guiExecFunc(*self._guiExecArgs)

  def _bg(self):
    success=False
    try:
      self._download()
    except:
      log.error("Chyba při stahování")
    else:
      if not self._shouldCancel:
        success=True
    if not success:
      self._guiExec(self._error)
      return
    self._guiExec(self._downloadSuccess)

  def _download(self):
    request = urllib.request.Request(self.downloadUrl)
    remote = urllib.request.urlopen(request)
    if remote.code != 200:
      raise RuntimeError("Stahování selhalo s chybou %d" % remote.code)
    size = int(remote.headers["content-length"])
    with open(self.downloadTarget, "wb") as local:
      self._guiExec(self._downloadReport, 0, size)
      read = 0
      chunk=8192
      while True:
        if self._shouldCancel:
          return
        if size -read <chunk:
          chunk =size -read
        block = remote.read(chunk)
        if not block:
          break
        read += len(block)
        if self._shouldCancel:
          return
        local.write(block)
        self._guiExec(self._downloadReport, read, size)
      if read < size:
        raise RuntimeError("Stahovaný obsah je příliš krátký")
    self._guiExec(self._downloadReport, read, size)

  def _downloadReport(self, read, size):
    if self._shouldCancel:
      return
    percent = int(float(read) / size * 100)
    cont, skip = self._progressDialog.Update(percent, _("Stahuji"))
    if not cont:
      self._shouldCancel = True
      self._stopped()

  def _stopped(self):
    self._guiExecTimer = None
    self._guiExecFunc = None
    self._guiExecArgs = None
    self._progressDialog.Hide()
    self._progressDialog.Destroy()
    self._progressDialog = None

  def _error(self):
    self._stopped()
    errorMessage = "Soubor nelze stáhnout."
    errorTitle = "Upozornění"
    wx.MessageBox(errorMessage, errorTitle)

  def _downloadSuccess(self):
    self._stopped()
    # start addon update
    os.startfile(self.downloadTarget)

    # end of download with progressbar


  def chooseNVDAObjectOverlayClasses(self, obj, clsList):
    if obj.windowClassName == "ListBox":
      clsList.insert(0, ClsRssChannelsList)

  def onRssReader(self, evt):
    gui.mainFrame.prePopup()
    d = RssReader(gui.mainFrame)
    d.Show()
    gui.mainFrame.postPopup()

  @script(
    description="Aktivuje okno RSS čtečky",
    gesture="kb:control+shift+i"
  )
  def script_displayRss(self, gesture):
    wx.CallAfter(self.onRssReader, None)


class ClsRssChannelsList(IAccessible):

  def reportFocus(self):
    messageList = []
    obj = api.getForegroundObject()
    title = obj.name
    if self.name:
      messageList.append(self.name)
      if title.startswith("Zprávy"):
        if automaticallyAnnounceDescription:
          messageList.append(currentArticleDescription)
      else:
        if title == mainTitle:
          if searchFilter:
            if currentChannelResults:
              messageList.append("Nalezených zpráv " + currentChannelResults)
          elif automaticallyAnnounceNumberofNews:
            if currentChannelNewMessages:
              messageList.append("Nových zpráv " + currentChannelNewMessages)
      ui.message(". ".join(messageList))
    else:
      super(ClsRssChannelsList,self).reportFocus()

  def event_stateChange(self):
    # only to force NVDA not saying "selected" after each list item
    pass
